<?php

/**
 * @file
 * Administrative forms for Autolink Tracking module.
 */

/**
 * Callback for link view form.
 */
function autolink_tracking_link_overview_page($link) {
  if (!empty($_POST['relationships'])) {
    $output = drupal_get_form('autolink_tracking_multiple_disable_link_confirm');
  }
  else {
    $args = explode('/', $link->link_path);
    $path = autolink_tracking_build_path($args[1]);
    drupal_set_title(t("Overview for keyword $link->keyword, linking to $path."));
    $output = drupal_get_form('autolink_tracking_link_overview', $link->lid);
  }
  return $output;
}

/**
 * Link overview form.
 */
function autolink_tracking_link_overview($lid) {
  $header = array(
    'node' => array('data' => t('Node'), 'field' => 'r.nid'),
    'title' => array('data' => t('Title'), 'field' => 'n.title'),
    'path' => array('data' => t('Path')),
    'relevance' => array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc'),
  );

  $query = db_select('autolink_link', 'a');
  $query->leftJoin('autolink_relationship', 'r', 'a.lid = r.lid');
  $query->leftJoin('node', 'n', 'r.nid = n.nid');
  $query->condition('r.relevance', 9999, '<>');

  $count_query = clone $query;
  $count_query->addExpression('COUNT(r.rel_id)');

  $query = $query->extend('PagerDefault')->extend('TableSort');
  $query
    ->fields('a', array('lid', 'keyword'), 'r', array('rel_id', 'nid', 'relevance'), 'n', array('title'))
    ->limit(25)
    ->orderByHeader($header)
    ->setCountQuery($count_query);
  $result = $query->execute();

  $options = array();
  foreach ($result as $link) {
    $paths = array();
    if (isset($link->nid)) {
      $path = autolink_tracking_build_path($link->nid);
      $options[$account->uid] = array(
        'node' => array('#value' => check_plain($link->nid)),
        'title' =>  array('#value' => l($link->title, "node/$link->nid")),
        'path' => array('#value' => l($path, 'node/'. $link->nid)),
        'relevance' => array('#value' => check_plain($link->relevance)),
      );
    }
  }

  $form['relationships'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No relationships available.'),
  );
  $form['pager'] = array('#markup' => theme('pager'));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Remove relationships'));

  return $form;
}

/**
 * Form validation for outgoing links update form.
 */
function autolink_tracking_link_overview_validate($form, &$form_state) {
  $form_state['values']['relationships'] = array_filter($form_state['values']['relationships']);
  if (!count($form_state['values']['relationships'])) {
    form_set_error('', t('No relationships were selected.'));
  }
}

/**
 * Theme the incoming links overview as a sortable list of links.
 *
 * @ingroup themeable
 * @see autolink_overview_incoming_links()
 */
function theme_autolink_tracking_link_overview($form) {
  $header = array(
    theme('table_select_header_cell'),
    array('data' => t('Node'), 'field' => 'r.nid', 'sort' => 'asc'),
    array('data' => t('Title'), 'field' => 'n.title', 'sort' => 'asc'),
    t('Path'),
    array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc'),
  );

  if (isset($form['node']) && is_array($form['node'])) {
    foreach (element_children($form['node']) as $key) {
      $rows[] = array(
        drupal_render($form['relationships'][$key]),
        drupal_render($form['node'][$key]),
        drupal_render($form['title'][$key]),
        drupal_render($form['path'][$key]),
        drupal_render($form['relevance'][$key]),
      );
    }
  }
  else {
    $rows[] = array(array('data' => t('No relationships available.'), 'colspan' => '5'));
  }

  $output = theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

/**
 * Confirmation form for removing relationships from the incoming links form.
 */
function autolink_tracking_multiple_disable_link_confirm(&$form_state) {
  $edit = $form_state['post'];
  $form['relationships'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);

  foreach (array_filter($edit['relationships']) as $rel_id => $value) {
    $link = db_query('SELECT a.lid, a.group_type, r.rel_id, r.nid FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE r.rel_id = :rel_id', array(':rel_id' => $rel_id))->fetchObject();
    $path = autolink_tracking_build_path($link->nid);
    $form['relationships'][$rel_id] = array(
      '#type' => 'hidden',
      '#value' => $rel_id,
      '#prefix' => '<li>',
      '#suffix' => check_plain($path) ."</li>\n",
    );
  }

  return confirm_form($form,
                      t('Are you sure you want to remove these relationships?'),
                      "admin/config/content/autolink/$link->group_type/link/$link->lid", t('This action cannot be undone.'),
                      t('Remove all'), t('Cancel'));
}

/**
 * Submit handler for removing links from the incoming links form.
 */
function autolink_tracking_multiple_disable_link_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    foreach ($form_state['values']['relationships'] as $rel_id => $value) {
      $link = db_query('SELECT a.lid, a.group_type, r.rel_id, r.nid FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE r.rel_id = :rel_id', array(':rel_id' => $rel_id))->fetchObject();
      autolink_tracking_remove_relationship($rel_id);
      $node = node_load($link->nid);
      if (autolink_node_process($node)) {
        node_save($node);
      }
    }
    drupal_set_message(t('The relationships have been removed.'));
  }

  $form_state['redirect'] = "admin/config/content/autolink/$link->group_type/link/$link->lid";

  return;
}
