<?php

/**
 * @file
 * Front end pages for Autolink Tracking module.
 */

/**
 * Callback function to build the related links page.
 */
function autolink_tracking_incoming_links_page($node) {
  if (!empty($_POST['relationships'])) {
    $output = drupal_get_form('autolink_tracking_multiple_disable_incoming_confirm');
  }
  else {
    drupal_set_title(t("Incoming links for $node->title"));
    $output = drupal_get_form('autolink_tracking_incoming_filter_form');
    $output .= drupal_get_form('autolink_tracking_overview_incoming_links', $node->nid);
  }
  return $output;
}

/**
 * Form builder for incoming links filter form.
 *
 * @ingroup forms
 */
function autolink_tracking_incoming_filter_form() {
  $session = isset($_SESSION['autolink_incoming_filter']) ? $_SESSION['autolink_incoming_filter'] : array();
  $filters = autolink_tracking_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only relationships where'),
    '#theme' => 'exposed_filters__user',
  );
  foreach ($session as $filter) {
    list($type, $value) = $filter;
    $value = $filters[$type]['options'][$value];
    $t_args = array('%property' => $filters[$type]['title'], '%value' => $value);
    if ($i++) {
      $form['filters']['current'][] = array('#markup' => t('and where %property is %value', $t_args));
    }
    else {
      $form['filters']['current'][] = array('#markup' => t('%property is %value', $t_args));
    }
  }

  $form['filters']['status'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('clearfix')),
    '#prefix' => ($i ? '<div class="additional-filters">' . t('and where') . '</div>' : ''),
  );
  $form['filters']['status']['filters'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('filters')),
  );
  foreach ($filters as $key => $filter) {
    $form['filters']['status']['filters'][$key] = array(
      '#type' => 'select',
      '#options' => $filter['options'],
      '#title' => $filter['title'],
      '#default_value' => '[any]',
    );
  }

  $form['filters']['status']['actions'] = array(
    '#type' => 'actions',
    '#attributes' => array('class' => array('container-inline')),
  );
  $form['filters']['status']['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => (count($session) ? t('Refine') : t('Filter')),
  );
  if (count($session)) {
    $form['filters']['status']['actions']['undo'] = array(
      '#type' => 'submit',
      '#value' => t('Undo'),
    );
    $form['filters']['status']['actions']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset'),
    );
  }

  drupal_add_library('system', 'drupal.form');

  return $form;
}

/**
 * Process result from user administration filter form.
 */
function autolink_tracking_incoming_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];
  $filters = autolink_tracking_filters();
  switch ($op) {
    case t('Filter'):
    case t('Refine'):
      // Apply every filter that has a choice selected other than 'any'.
      foreach ($filters as $filter => $options) {
        if (isset($form_state['values'][$filter]) && $form_state['values'][$filter] != '[any]') {
          $options = $filters[$filter]['options'];
          // Only accept valid selections offered on the dropdown, block bad input.
          if (isset($options[$form_state['values'][$filter]])) {
            $_SESSION['autolink_incoming_filter'][] = array($filter, $form_state['values'][$filter]);
          }
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION['autolink_incoming_filter']);
      break;
    case t('Reset'):
      $_SESSION['autolink_incoming_filter'] = array();
      break;
    case t('Update'):
      return;
  }

  $form_state['redirect'] = 'node/'. arg(1) .'/autolink/incoming';
  return;
}

/**
 * Form builder incoming links overview form.
 *
 * @ingroup forms
 */
function autolink_tracking_overview_incoming_links() {

  $header = array(
    'type' => array('data' => t('Type'), 'field' => 'a.type'),
    'group_type' => array('data' => t('Link group'), 'field' => 'a.group_type'),
    'keyword' => array('data' => t('Keyword'), 'field' => 'a.keyword'),
    'origin' => array('data' => t('Origin')),
    'relevance' => array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc'),
    'operations' => array('data' => t('Operations')),
  );

  $groups = autolink_ui_get_groups();

  $query = db_select('autolink_link', 'a');
  $query->leftJoin('autolink_relationship', 'r', 'a.lid = r.lid');
  $query->condition('r.relevance', 9999, '<>');
  autolink_tracking_build_filter_query($query, 'incoming');

  $count_query = clone $query;
  $count_query->addExpression('COUNT(r.rel_id)');

  $query = $query->extend('PagerDefault')->extend('TableSort');
  $query
    ->fields('a', array('lid', 'type', 'group_type', 'keyword', 'link_path'), 'r', array('rel_id', 'nid', 'relevance'))
    ->limit(25)
    ->orderByHeader($header)
    ->setCountQuery($count_query);
  $result = $query->execute();

  $destination = drupal_get_destination();

  foreach ($result as $link) {
    $path = autolink_tracking_build_path($link->nid);
    $options[$link->lid] = array(
      'type' => array('#value' => check_plain($link->type)),
      'group_type' =>  array('#value' => l($groups[$link->group_type]->name, "admin/config/content/autolink/$link->group_type")),
      'keyword' => array('#value' => l($link->keyword, "admin/config/content/autolink/$link->group_type/link/$link->lid")),
      'origin' => array('#value' => l($path, 'node/'. $link->nid)),
      'relevance' =>  array('#value' => $link->relevance),
      'operations' => array('data' => array('#type' => 'link', '#title' => t('edit'), '#href' => "admin/config/content/$link->group_type/edit/link/$link->lid", '#options' => array('query' => $destination))),
    );
  }

  $form['relationships'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No relationships available.'),
  );
  $form['pager'] = array('#markup' => theme('pager'));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Remove relationships'));

  return $form;
}

/**
 * Form validation for outgoing links update form.
 */
function autolink_tracking_overview_incoming_links_validate($form, &$form_state) {
  $form_state['values']['relationships'] = array_filter($form_state['values']['relationships']);
  if (!count($form_state['values']['relationships'])) {
    form_set_error('', t('No links were selected.'));
  }
}

/**
 * Confirmation form for removing relationships from the incoming links form.
 */
function autolink_tracking_multiple_disable_incoming_confirm(&$form_state) {
  $edit = $form_state['post'];
  $nid = arg(1);
  $form['relationships'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);

  foreach (array_filter($edit['relationships']) as $rel_id => $value) {
    $relationship = db_query('SELECT r.rel_id, r.nid FROM {autolink_relationship} r WHERE r.rel_id = :rel_id', array(':rel_id' => $rel_id))->fetchObject();
    $path = autolink_tracking_build_path($relationship->nid);
    $form['relationships'][$rel_id] = array(
      '#type' => 'hidden',
      '#value' => $rel_id,
      '#prefix' => '<li>',
      '#suffix' => check_plain($path) ."</li>\n",
    );
  }

  return confirm_form($form,
                      t('Are you sure you want to remove these relationships?'),
                      "node/$nid/autolink/incoming", t('This action cannot be undone.'),
                      t('Remove all'), t('Cancel'));
}

/**
 * Submit handler for removing links from the incoming links form.
 */
function autolink_tracking_multiple_disable_incoming_confirm_submit($form, &$form_state) {
  $nid = arg(1);
  if ($form_state['values']['confirm']) {
    foreach ($form_state['values']['relationships'] as $rel_id => $value) {
      $link = db_query('SELECT a.lid, r.rel_id, r.nid FROM {autolink_link} a LEFT JOIN {autolink_relationship} r ON a.lid = r.lid WHERE r.rel_id = :rel_id', array(':rel_id' => $rel_id))->fetchObject();
      autolink_tracking_remove_relationship($rel_id);

      // Remove disabled links from the originating node.
      $node = node_load($link->nid);
      if (autolink_node_process($node)) {
        node_save($node);
      }
    }
    drupal_set_message(t('The relationships have been removed.'));
  }

  $form_state['redirect'] = "node/$nid/autolink/incoming";

  return;
}

/**
 * Callback function to build the outgoing related links page.
 */
function autolink_tracking_outgoing_links_page($node) {
  if (!empty($_POST['relationships'])) {
    $output = drupal_get_form('autolink_tracking_multiple_disable_outgoing_confirm');
  }
  else {
    drupal_set_title(t("Outgoing links for $node->title"));
    $output = drupal_get_form('autolink_tracking_outgoing_filter_form');
    $output .= drupal_get_form('autolink_tracking_overview_outgoing_links', $node->nid);
  }
  return $output;
}
/**
 * Form builder for outgoing links filter form.
 *
 * @ingroup forms
 */
function autolink_tracking_outgoing_filter_form() {
  $session = isset($_SESSION['autolink_outgoing_filter']) ? $_SESSION['autolink_outgoing_filter'] : array();
  $filters = autolink_tracking_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only relationships where'),
    '#theme' => 'exposed_filters__user',
  );
  foreach ($session as $filter) {
    list($type, $value) = $filter;
    $value = $filters[$type]['options'][$value];
    $t_args = array('%property' => $filters[$type]['title'], '%value' => $value);
    if ($i++) {
      $form['filters']['current'][] = array('#markup' => t('and where %property is %value', $t_args));
    }
    else {
      $form['filters']['current'][] = array('#markup' => t('%property is %value', $t_args));
    }
  }

  $form['filters']['status'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('clearfix')),
    '#prefix' => ($i ? '<div class="additional-filters">' . t('and where') . '</div>' : ''),
  );
  $form['filters']['status']['filters'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('filters')),
  );
  foreach ($filters as $key => $filter) {
    $form['filters']['status']['filters'][$key] = array(
      '#type' => 'select',
      '#options' => $filter['options'],
      '#title' => $filter['title'],
      '#default_value' => '[any]',
    );
  }

  $form['filters']['status']['actions'] = array(
    '#type' => 'actions',
    '#attributes' => array('class' => array('container-inline')),
  );
  $form['filters']['status']['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => (count($session) ? t('Refine') : t('Filter')),
  );
  if (count($session)) {
    $form['filters']['status']['actions']['undo'] = array(
      '#type' => 'submit',
      '#value' => t('Undo'),
    );
    $form['filters']['status']['actions']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset'),
    );
  }

  drupal_add_library('system', 'drupal.form');

  return $form;
}

/**
 * Process result from outgoing links filter form.
 */
function autolink_tracking_outgoing_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];
  $filters = autolink_tracking_filters();
  switch ($op) {
    case t('Filter'):
    case t('Refine'):
      // Apply every filter that has a choice selected other than 'any'.
      foreach ($filters as $filter => $options) {
        if (isset($form_state['values'][$filter]) && $form_state['values'][$filter] != '[any]') {
          $options = $filters[$filter]['options'];
          // Only accept valid selections offered on the dropdown, block bad input.
          if (isset($options[$form_state['values'][$filter]])) {
            $_SESSION['autolink_outgoing_filter'][] = array($filter, $form_state['values'][$filter]);
          }
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION['autolink_outgoing_filter']);
      break;
    case t('Reset'):
      $_SESSION['autolink_outgoing_filter'] = array();
      break;
    case t('Update'):
      return;
  }

  $form_state['redirect'] = 'node/'. arg(1) .'/autolink/outgoing';
  return;
}

/**
 * Form builder; outgoing links overview form.
 *
 * @ingroup forms
 */
function autolink_tracking_overview_outgoing_links() {
  $header = array(
    'type' => array('data' => t('Type'), 'field' => 'a.type'),
    'group_type' => array('data' => t('Link group'), 'field' => 'a.group_type'),
    'keyword' => array('data' => t('Keyword'), 'field' => 'a.keyword'),
    'destination' => array('data' => t('Destination')),
    'relevance' => array('data' => t('Relevance'), 'field' => 'r.relevance', 'sort' => 'desc'),
    'operations' => array('data' => t('Operations')),
  );

  $groups = autolink_ui_get_groups();

  $query = db_select('autolink_link', 'a');
  $query->leftJoin('autolink_relationship', 'r', 'a.lid = r.lid');
  $query->condition('r.relevance', 9999, '<>');
  autolink_tracking_build_filter_query($query, 'outgoing');
a.lid, a.type, a.group_type, a.keyword, a.link_path, r.rel_id, r.relevance
  $count_query = clone $query;
  $count_query->addExpression('COUNT(r.rel_id)');

  $query = $query->extend('PagerDefault')->extend('TableSort');
  $query
    ->fields('a', array('lid', 'type', 'group_type', 'keyword', 'link_path'), 'r', array('rel_id', 'relevance'))
    ->limit(25)
    ->orderByHeader($header)
    ->setCountQuery($count_query);
  $result = $query->execute();

  $destination = drupal_get_destination();

  foreach ($result as $link) {
    $destination = autolink_ui_get_path($link->link_path);
    $options[$link->lid] = array(
      'type' => array('#value' => check_plain($link->type)),
      'group_type' =>  array('#value' => l($groups[$link->group_type]->name, "admin/config/content/autolink/$link->group_type")),
      'keyword' => array('#value' => l($link->keyword, "admin/config/content/autolink/$link->group_type/link/$link->lid")),
      'destination' => array('#value' => l($destination, $link->link_path)),
      'relevance' =>  array('#value' => $link->relevance),
      'operations' => array('data' => array('#type' => 'link', '#title' => t('edit'), '#href' => "admin/config/content/$link->group_type/edit/link/$link->lid", '#options' => array('query' => $destination))),
    );
  }

  $form['relationships'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No relationships available.'),
  );
  $form['pager'] = array('#markup' => theme('pager'));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Remove relationships'));

  return $form;
}

/**
 * Form validation for outgoing links update form.
 */
function autolink_tracking_overview_outgoing_links_validate($form, &$form_state) {
  $form_state['values']['relationships'] = array_filter($form_state['values']['relationships']);
  if (!count($form_state['values']['relationships'])) {
    form_set_error('', t('No links were selected.'));
  }
}

/**
 * Confirmation form for removing relationships from the outgoing links form.
 */
function autolink_tracking_multiple_disable_outgoing_confirm(&$form_state) {
  $edit = $form_state['post'];

  $form['relationships'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);
  foreach (array_filter($edit['relationships']) as $rel_id => $value) {
    $relationship = db_query('SELECT a.keyword, a.argument, r.nid FROM {autolink_relationship} r LEFT JOIN {autolink_link} a ON r.lid = a.lid WHERE r.rel_id = :rel_id', array(':rel_id' => $rel_id))->fetchObject();
    $form['relationships'][$rel_id] = array(
      '#type' => 'hidden',
      '#value' => $rel_id,
      '#prefix' => '<li>',
      '#suffix' => check_plain($relationship->argument) ."</li>\n",
    );
  }

  return confirm_form($form,
                      t('Are you sure you want to remove these relationships?'),
                      "node/$nid/autolink/outgoing", t('This action cannot be undone.'),
                      t('Remove all'), t('Cancel'));
}

/**
 * Submit handler for removing relationships from the outgoing links form.
 */
function autolink_tracking_multiple_disable_outgoing_confirm_submit($form, &$form_state) {
  $nid = arg(1);
  if ($form_state['values']['confirm']) {
    foreach ($form_state['values']['relationships'] as $rel_id => $value) {
      autolink_tracking_remove_relationship($rel_id);
    }

    // Process the node to remove the disabled links.
    $node = node_load($nid);
    if (autolink_node_process($node)) {
      node_save($node);
    }
    drupal_set_message(t('The relationships have been removed.'));
  }

  $form_state['redirect'] = "node/$nid/autolink/outgoing";

  return;
}

/**
 * Defines filters for outgoing links filter form.
 */
function autolink_tracking_filters() {
  $filters = array();

  // Link types filter.
  $link_types = autolink_ui_link_types();
  if (count($link_types)) {
    $filters['link_type'] = array(
      'title' => t('link type'),
      'where' => "a.type = '%s'",
      'options' => $link_types,
      'join' => '',
    );
  }

  // Groups filter.
  $groups = array();
  foreach (autolink_ui_get_groups() as $group) {
    $groups[$group->type] = $group->name;
  }

  if (count($groups)) {
    $filters['link_group'] = array(
      'title' => t('link group'),
      'where' => "a.group_type = '%s'",
      'options' => $groups,
      'join' => '',
    );
  }

  return $filters;
}

/**
 * Build query for autolink admin filters based on session.
 */
function autolink_tracking_build_filter_query(SelectQuery $query, $dir) {
  $filters = autolink_tracking_filters();

  // Extend Query with filter conditions.
  foreach (isset($_SESSION['autolink_'. $dir .'_filter']) ? $_SESSION['autolink_'. $dir .'_filter'] : array() as $filter) {
    list($key, $value) = $filter;
    $query->condition($filters[$key]['field'], $value);
  }
}
