<?php

/**
 * @file
 * Administrative page callbacks for the autolink module.
 */

/**
 * Settings form for autolink.
 *
 * @ingroup forms
 * @see autolink_form_settings_submit()
 */
function autolink_ui_form_settings($form, &$form_state) {
  $form['general_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('General settings'),
    '#collapsible' => TRUE,
    '#weight' => -10,
  );

  // General settings.
  $form['general_settings']['autolink_link_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Enable link types'),
    '#options' => autolink_ui_link_types(),
    '#default_value' => array_keys(autolink_ui_setting('autolink_link_types', autolink_ui_link_types(), 1)),
    '#description' => t('Note: disabling a link type will remove all link definitions '.
                        'from that link type.'),
  );

  // Add pattern processing with Token for link titles.
  $form['general_settings']['autolink_tokens'] = array(
    '#type' => 'item',
    '#value' => t('Title patterns are used to generate link text when generating '.
                  'links in blocks or as part of the <i>title</i> attribute in links '.
                  'that are generated in node content.'));

  if (variable_get('autolink_link_types_node', 1) == AUTOLINK_SETTING_ENABLED) {
    $form['general_settings']['autolink_node_title'] = array(
      '#type' => 'textfield',
      '#title' => t('Pattern replacement for node links'),
      '#default_value' => variable_get('autolink_node_title', '[title]'),
      '#description' => t('Title patterns are used to generate link text when generating '.
                          'links in blocks or as part of the <i>title</i> attribute in links '.
                          'that are generated in node content.'),
    );
    $form['general_settings']['node_token_help'] = array(
      '#type' => 'fieldset',
      '#title' => t('Replacement patterns'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['general_settings']['node_token_help']['node_help'] = array(
      '#theme' => 'token_tree',
      '#token_types' => array('node'),
    );
  }

  if (variable_get('autolink_link_types_user', 1) == AUTOLINK_SETTING_ENABLED) {
    $form['general_settings']['autolink_user_title'] = array(
      '#type' => 'textfield',
      '#title' => t('Pattern replacement for user links'),
      '#default_value' => variable_get('autolink_user_title', '[user]'),
    );
    $form['general_settings']['user_token_help'] = array(
      '#type' => 'fieldset',
      '#title' => t('Replacement patterns'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['general_settings']['user_token_help']['user_help'] = array(
      '#theme' => 'token_tree',
       '#token_types' => array('user'),
    );
  }

  if (variable_get('autolink_link_types_taxonomy', 1) == AUTOLINK_SETTING_ENABLED) {
    $form['general_settings']['autolink_taxonomy_title'] = array(
      '#type' => 'textfield',
      '#title' => t('Pattern replacement for taxonomy term links'),
      '#default_value' => variable_get('autolink_taxonomy_title', '[term]'),
    );
    $form['general_settings']['taxonomy_token_help'] = array(
      '#type' => 'fieldset',
      '#title' => t('Replacement patterns'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['general_settings']['taxonomy_token_help']['taxonomy_help'] = array(
      '#theme' => 'token_tree',
      '#token_types' => array('term'),
    );
  }

  if (variable_get('autolink_link_types_node', 1) == AUTOLINK_SETTING_ENABLED) {
    $form['general_settings']['autolink_link_tagging'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enable link tagging on node edit forms'),
      '#default_value' => variable_get('autolink_link_tagging', 1),
      '#description' => t('With link tagging enabled, link definitions that link to a given node '.
                          'can be managed from the node edit form. This setting only applies to '.
                          '<i>node</i> links.'),
    );
  }

  $form['general_settings']['autolink_display_alias'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display aliases on link overview form'),
    '#default_value' => variable_get('autolink_display_alias', 1),
    '#description' => t('Organize the link overview form by displaying aliases in the '.
                        'destination field. Note: Autolink will always use normal Drupal '.
                        'paths in building links.'),
  );

  $defaults = autolink_ui_get_settings();

  if (module_exists('autolink_node') || module_exists('autolink_comment')) {
    $form['global'] = array(
      '#type' => 'fieldset',
      '#title' => t('Global link generation settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#weight' => -9,
      '#description' => t('Global settings allow site administrators more control over '.
                          'the location and frequency of link generation in node content.'),
    );
    $form['global']['autolink_link_limit'] = array(
      '#type' => 'select',
      '#title' => t('Limit links per keyword per node'),
      '#options' => array(
        'all' => 'all',
        '1' => '1',
        '2' => '2',
        '3' => '3',
        '4' => '4',
        '5' => '5',
        '6' => '6',
        '7' => '7',
        '8' => '8',
        '9' => '9',
        '10' => '10',
      ),
      '#default_value' => variable_get('autolink_link_limit', 'all'),
      '#description' => t('Providing a maximum number of links allowed will ensure that only that number '.
                          'of links for a given keyword will be generated per node.'),
    );
    $form['global']['autolink_total_link_limit'] = array(
      '#type' => 'select',
      '#title' => t('Limit total links generated per node'),
      '#options' => array(
        'all' => 'all',
        '1' => '1',
        '2' => '2',
        '3' => '3',
        '4' => '4',
        '5' => '5',
        '6' => '6',
        '7' => '7',
        '8' => '8',
        '9' => '9',
        '10' => '10',
        '11' => '11',
        '12' => '12',
        '13' => '13',
        '14' => '14',
        '15' => '15',
        '16' => '16',
        '17' => '17',
        '18' => '18',
        '19' => '19',
        '20' => '20',
      ),
      '#default_value' => variable_get('autolink_total_link_limit', 'all'),
      '#description' => t('Limit the total number of links generated by Autolink on any given node.'),
    );
    $form['global']['autolink_remove_links'] = array(
      '#type' => 'checkbox',
      '#title' => t('Remove links during node edit'),
      '#default_value' => variable_get('autolink_remove_links', 1),
    );
  }

  // Automatic link generation settings.
  $form['automatic'] = array(
    '#type' => 'fieldset',
    '#title' => t('Automatic link generation'),
    '#weight' => -8,
    '#collapsible' => TRUE,
    '#collapsed' => variable_get('autolink_auto_generate', 0) == 1 ? FALSE : TRUE,
  );
  $form['automatic']['autolink_auto_generate'] = array(
    '#type' => 'checkbox',
    '#title' => t('Automatically generate links from node titles, users, or terms'),
    '#default_value' => variable_get('autolink_auto_generate', 0),
    '#description' => t('Automatically generate links when nodes, users, or terms '.
                        'are located in node content.<br />' .
                        'Note: Link type and node type settings are available when '.
                        'this setting is enabled.'),
  );

  if (variable_get('autolink_auto_generate', 0) == AUTOLINK_SETTING_ENABLED) {
    if (module_exists('taxonomy')) {
      $type_options = array('node' => 'Node', 'user' => 'User', 'taxonomy' => 'Taxonomy Term');
    }
    else {
      $type_options = array('node' => 'Node', 'user' => 'User');
    }

    $form['automatic']['autolink_auto_link_types'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Limit automatic generation by link type'),
      '#options' => $type_options,
      '#default_value' => array_keys(autolink_ui_setting('autolink_auto_link_types', autolink_ui_link_types(), 0)),
    );

    if (variable_get('autolink_auto_link_types_node', 0) == AUTOLINK_SETTING_ENABLED) {
      $form['automatic']['autolink_auto_node_types'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Limit automatic link generation by content type'),
        '#options' => node_type_get_names(),
        '#default_value' => array_keys(autolink_ui_setting('autolink_auto_node_types', node_type_get_names(), 0)),
        '#description' => t('Select which content types automatic links may be applied to.'),
      );
    }

    if (variable_get('autolink_auto_link_type_user', 0) == AUTOLINK_SETTING_ENABLED) {
      $form['automatic']['autolink_auto_user_roles'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Limit automatic link destinations by user role'),
        '#options' => user_roles(),
        '#default_value' => array_keys(autolink_ui_setting('autolink_auto_user_roles', user_roles(), 0)),
        '#description' => t('User roles that are not selected will not be considered '.
                            'for automatic link generation. i.e. If <i>administrator</i> is not '.
                            'selected then Autolink will not seek out user name under the '.
                            'role of <i>administrator</i>'),
      );
    }
    if (variable_get('autolink_auto_link_type_taxonomy', 0) == AUTOLINK_SETTING_ENABLED) {
      $form['automatic']['autolink_auto_vocabularies'] = array(
        '#type' => 'select',
        '#title' => t('Limit autolink link detection by vocabulary'),
        '#multiple' => TRUE,
        '#options' => autolink_ui_get_vocabularies(),
        '#default_value' => array_keys(autolink_ui_setting('autolink_auto_vocabularies', autolink_ui_get_vocabularies(), 0)),
        '#description' => t('Terms within the selected vocabularies will be sought in '.
                            'node content.'),
      );
    }
  }

  // Default link settings.
  $form['default'] = array(
    '#type' => 'fieldset',
    '#title' => t('Link attributes'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => -7,
    '#description' => t('Default settings will be used to populate new link definition forms and '.
                        'when generating links in node content.'),
  );
  $form['default']['autolink_case_sensitivity'] = array(
    '#type' => 'checkbox',
    '#title' => t('Case-sensitivity'),
    '#default_value' => $defaults->case_sensitivity,
    '#description' => t('Keywords will only be replaced with links if they match the case of the keyword. '.
                        'This setting serves only as a default, as the case-sensitivity attribute can '.
                        'be managed on a link-by-link basis on the link add/edit form.'),
  );
  $form['default']['autolink_style'] = array(
    '#type' => 'textfield',
    '#title' => t('Style'),
    '#default_value' => $defaults->style,
    '#description' => t('Although it is recommended that you use a stylesheet and class, inline styles may '.
                        'be used for additional link attributes.'),
  );
  $form['default']['autolink_class'] = array(
    '#type' => 'textfield',
    '#title' => t('Class'),
    '#default_value' => $defaults->class,
    '#description' => t('New link definitions can automatically be assigned classes. To assign more than '.
                        'one class enter a space separated list.'),
  );
  $form['default']['autolink_internal_target'] = array(
    '#type' => 'select',
    '#title' => t('Target (internal links)'),
    '#options' => array(
      'none' => 'none',
      '_blank' => '_blank',
      '_parent' => '_parent',
      '_self' => '_self',
      '_top' => '_top',
    ),
    '#default_value' => $defaults->internal_target,
    '#description' => t('Specifies where to open internally linked documents.'),
  );
  $form['default']['autolink_external_target'] = array(
    '#type' => 'select',
    '#title' => t('Target (external links)'),
    '#options' => array(
      'none' => 'none',
      '_blank' => '_blank',
      '_parent' => '_parent',
      '_self' => '_self',
      '_top' => '_top',
    ),
    '#default_value' => $defaults->external_target,
    '#description' => t('Specifies where to open externally linked documents.'),
  );
  $form['default']['autolink_external_relationship'] = array(
    '#type' => 'checkbox',
    '#title' => t('Set <i>rel</i> attribute to <i>nofollow</i> for external links'),
    '#default_value' => variable_get('autolink_external_relationship', 1),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['buttons']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset to defaults'),
    '#weight' => 10,
  );

  return $form;
}

/**
 * Validation handler for the Autolink settings form.
 */
function autolink_ui_form_settings_validate($form, &$form_state) {
  foreach ($form_state['values']['autolink_link_types'] as $type => $set) {
    if ($set) {
      if ($type == 'search' && !module_exists('search')) {
        form_set_error('autolink_link_types', t('You must enable the search module before enabling the <i>search</i> link type.'));
      }
      elseif ($type == 'taxonomy' && !module_exists('taxonomy')) {
        form_set_error('autolink_link_types', t('You must enable the taxonomy module before enabling the <i>taxonomy</i> link type.'));
      }
    }
  }
}

/**
 * Submit handler for Autolink settings form.
 *
 * This handler sets miltiple value field values by creating individual variables
 * for each value. This makes node and link type settings much easier to work with
 * on other forms and in processing functions and is cleaner than using the
 * 'array_filter' field.
 */
function autolink_ui_form_settings_submit($form, &$form_state) {
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : '';

  // Exclude unnecessary elements.
  unset($form_state['values']['submit'], $form_state['values']['reset'], $form_state['values']['form_id'], $form_state['values']['op'], $form_state['values']['form_token'], $form_state['values']['form_build_id']);

  foreach ($form_state['values'] as $key => $value) {
    // The form was reset to defaults. Delete all variables.
    if ($op == t('Reset to defaults')) {
      if ($form_state['values'][$key]['#type'] == 'checkboxes') {
        foreach ($form_state['values'][$key] as $type => $values) {
          variable_del($key .'_'. $type);
        }
      }
      elseif ($key == 'autolink_auto_vocabularies') {
        foreach (autolink_ui_get_vocabularies() as $vid => $name) {
          variable_del('autolink_auto_vocabularies_'. $vid);
        }
      }
      else {
        variable_del($key);
      }
    }
    else {
      // Set checkboxes variables.
      if ($key == 'autolink_link_types' || $key == 'autolink_auto_link_types' || $key == 'autolink_auto_node_types' || $key == 'autolink_auto_user_roles') {
        foreach ($form_state['values'][$key] as $type => $state) {
          $variable = $key .'_'. $type;
          if ($state) {
            variable_set($variable, 1);
          }
          // Remove link definitions from a link group if that link group was disabled.
          elseif ($key == 'autolink_link_types' && variable_get($variable, 1) == AUTOLINK_SETTING_ENABLED) {
            $result = db_query("SELECT lid FROM {autolink_link} WHERE type = '%s'", $type);
            while ($lid = db_result($result)) {
              autolink_ui_link_delete($lid);
            }
            variable_set($variable, 0);
          }
          else {
            variable_set($variable, 0);
          }
        }
      }
      // Set taxonomy variables.
      elseif ($key == 'autolink_auto_vocabularies') {
        foreach (autolink_ui_get_vocabularies() as $vid => $name) {
          variable_set('autolink_auto_vocabularies_'. $vid, 0);
        }
        foreach ($form_state['values']['autolink_auto_vocabularies'] as $type => $state) {
          variable_set('autolink_auto_vocabularies_'. $vid, 1);
        }
      }
      else {
        variable_set($key, $value);
      }
    }
  }

  if ($op == t('Reset to defaults')) {
    drupal_set_message(t('The configuration options have been reset to their default values.'));
  }
  else {
    drupal_set_message(t('The configuration options have been saved.'));
  }

  cache_clear_all();
  drupal_theme_rebuild();
}

/**
 * Callback for displaying the groups overview form.
 */
function autolink_ui_admin_groups($callback_arg = '') {
  module_load_include('inc', 'autolink_ui', 'includes/autolink_ui.filters');
  module_load_include('inc', 'autolink_ui', 'includes/autolink_ui.operations');
  if (!empty($_POST['groups']) && isset($_POST['operation']) && ($_POST['operation'] == 'delete')) {
    $build['autolink_ui_group_multiple_delete_confirm'] = drupal_get_form('autolink_ui_group_multiple_delete_confirm');
  }
  else {
    $build['autolink_ui_filter_form'] = drupal_get_form('autolink_ui_filter_form');
    $build['autolink_ui_overview_groups'] = drupal_get_form('autolink_ui_overview_groups');
  }
  return $build;
}

/**
 * Form builder for Autolink groups overview filters.
 *
 * @ingroup forms
 * @see autolink_filter_form_submit()
 */
function autolink_ui_filter_form() {
  $session = isset($_SESSION['autolink_overview_filter']) ? $_SESSION['autolink_overview_filter'] : array();
  $filters = autolink_ui_filters();

  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only link groups where'),
    '#theme' => 'exposed_filters__group',
  );
  foreach ($session as $filter) {
    list($type, $value) = $filter;
    $value = $filters[$type]['options'][$value];
    $t_args = array('%property' => $filters[$type]['title'], '%value' => $value);
    if ($i++) {
      $form['filters']['current'][] = array('#markup' => t('and where %property is %value', $t_args));
    }
    else {
      $form['filters']['current'][] = array('#markup' => t('%property is %value', $t_args));
    }
  }

  $form['filters']['status'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('clearfix')),
    '#prefix' => ($i ? '<div class="additional-filters">' . t('and where') . '</div>' : ''),
  );
  $form['filters']['status']['filters'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('filters')),
  );
  foreach ($filters as $key => $filter) {
    $form['filters']['status']['filters'][$key] = array(
      '#type' => 'select',
      '#options' => $filter['options'],
      '#title' => $filter['title'],
      '#default_value' => '[any]',
    );
  }

  $form['filters']['status']['actions'] = array(
    '#type' => 'actions',
    '#attributes' => array('class' => array('container-inline')),
  );
  $form['filters']['status']['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => (count($session) ? t('Refine') : t('Filter')),
  );
  if (count($session)) {
    $form['filters']['status']['actions']['undo'] = array(
      '#type' => 'submit',
      '#value' => t('Undo'),
    );
    $form['filters']['status']['actions']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset'),
    );
  }

  drupal_add_library('system', 'drupal.form');

  return $form;
}

/**
 * Process result from autolink administration filter form.
 */
function autolink_ui_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];
  $filters = autolink_ui_filters();

  switch ($op) {
    case t('Filter'):
    case t('Refine'):
      foreach ($filters as $filter => $options) {
        if (isset($form_state['values']['filter'])) {
          $filter = $form_state['values']['filter'];
          $options = $filters[$filter]['options'];
          if (isset($options[$form_state['values'][$filter]])) {
            $_SESSION['autolink_overview_filter'][] = array($filter, $form_state['values'][$filter]);
          }
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION['autolink_overview_filter']);
      break;
    case t('Reset'):
      $_SESSION['autolink_overview_filter'] = array();
      break;
    case t('Update'):
      return;
  }

  $form_state['redirect'] = 'admin/config/content/autolink/';
  return;
}

/**
 * Form builder; Groups overview page.
 *
 * @ingroup forms
 */
function autolink_ui_overview_groups() {
  $header = array(
    'name' => array('data' => t('Group'), 'field' => 'g.name'),
    'description' => array('data' => t('Description'), 'field' => 'g.description'),
    'node_types' => array('data' => t('Node types')),
    'status' => array('data' => t('Status'), 'field' => 'g.status', 'sort' => 'desc'),
    'operations' => array('data' => t('Operations')),
  );

  $query = db_select('autolink_group', 'g');
  $query->leftJoin('autolink_group_node_type', 't', 'g.type = t.group_type');
  autolink_ui_build_filter_query($query);

  $count_query = clone $query;
  $count_query->addExpression('COUNT(g.type)');

  $query = $query->extend('PagerDefault')->extend('TableSort');
  $query
    ->fields('g', array('type', 'name', 'description', 'status'), 't', array('node_type'))
    ->limit(25)
    ->orderByHeader($header)
    ->setCountQuery($count_query);
  $result = $query->execute();

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#attributes' => array('class' => array('container-inline')),
  );

  // Add our own operations first.
  $options = array();
  $ops = autolink_ui_group_operations();
  foreach ($ops as $operation => $array) {
    $options[$operation] = $array['label'];
  }

  // Invoke hook_autolink_group_operations() to get a list of external operations.
  foreach (module_invoke_all('autolink_group_operations') as $operation => $array) {
    $options[$operation] = $array['label'];
  }

  $form['options']['operation'] = array(
    '#type' => 'select',
    '#title' => t('Operation'),
    '#title_display' => 'invisible',
    '#options' => $options,
    '#default_value' => 'activate',
  );
  $options = array();
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  $destination = drupal_get_destination();

  $status = array(t('disabled'), t('active'));
  $groups = array();
  foreach ($result as $group) {
    $node_types = array();
    $types_result = db_query('SELECT n.type, n.name FROM {node_type} n INNER JOIN {autolink_group_node_type} t ON n.type = t.node_type WHERE t.group_type = :type', array(':type' => $group->type));
    foreach ($types_result as $node_type) {
      $node_types[$node_type->type] = $node_type->name;
    }
    asort($node_types);

    $options[$group->type] = array(
      'name' => array('data' => array('#type' => 'link', '#title' => $group->name, '#href' => "admin/config/content/autolink/$group->type")),
      'description' => check_plain($group->description),
      'node_types' => theme('item_list', array('items' => $node_types)),
      'status' => $status[$group->status],
      'operations' => array('data' => array('#type' => 'link', '#title' => t('edit group'), '#href' => "admin/config/content/autolink/edit/group/$group->type", '#options' => array('query' => $destination))),
    );
  }

  $form['groups'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No groups available.'),
  );
  $form['pager'] = array('#markup' => theme('pager'));

  return $form;
}

/**
 * Form validation for Autolink groups administration update form.
 */
function autolink_ui_overview_groups_validate($form, &$form_state) {
  $form_state['values']['groups'] = array_filter($form_state['values']['groups']);
  if (!count($form_state['values']['groups'])) {
    form_set_error('', t('No groups were selected.'));
  }
}

/**
 * Submit handler for Autolink groups administration update form.
 */
function autolink_ui_overview_groups_submit($form, &$form_state) {
  $operations = autolink_ui_group_operations($form_state);
  $operations += module_invoke_all('autolink_group_operations', $form_state);
  $operation = $operations[$form_state['values']['operation']];
  // Filter out unchecked links.
  $groups = array_filter($form_state['values']['groups']);
  if ($function = $operation['callback']) {
    // Add in callback arguments if present.
    if (isset($operation['callback arguments'])) {
      $args = array_merge(array($groups), $operation['callback arguments']);
    }
    else {
      $args = array($groups);
    }
    // Call the function based on the operation that was selected.
    call_user_func_array($function, $args);

    drupal_set_message(t('The update has been performed.'));
  }
}

/**
 * Display form for adding and editing link groups.
 *
 * @ingroup forms
 * @see autolink_form_group_submit()
 */
function autolink_ui_form_group($form, &$form_state, $edit = array()) {
  if (!isset($form_state['group'])) {
    $group = is_object($edit) ? $edit : (object) $edit;
    $defaults = array(
      'status' => 1,
      'name' => '',
      'type' => '',
      'description' => '',
      'in_content' => 0,
      'types' => array(),
    );
    foreach ($defaults as $key => $value) {
      if (!isset($group->$key)) {
        $group->$key = $value;
      }
    }
    $form_state['group'] = $group;
  }
  else {
    $group = $form_state['group'];
  }

  $form['#group'] = $form_state['group'];

  // Check whether we need a deletion confirmation form.
  if (isset($form_state['confirm_delete']) && isset($form_state['values']['type'])) {
    return autolink_ui_group_confirm_delete($form, $form_state, $form_state['values']['type']);
  }

  $form['identification'] = array(
    '#type' => 'fieldset',
    '#title' => t('Identification'),
    '#collapsible' => TRUE,
  );
  $form['identification']['status'] = array(
    '#type' => 'select',
    '#title' => t('Status'),
    '#default_value' => $group->status,
    '#options' => array(0 => t('Disabled'), 1 => t('Active')),
  );
  $form['identification']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Group name'),
    '#default_value' => $group->name,
    '#maxlength' => 255,
    '#description' => t('The human-readable name for this link group.'),
    '#required' => TRUE,
  );
  $form['identification']['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => $group->description,
    '#description' => t('Description of the link group.'),
  );
  $form['identification']['in_content'] = array(
    '#type' => 'checkbox',
    '#title' => t('Generate links from this group in node content'),
    '#default_value' => $group->in_content,
    '#description' => t('Autolink will search for keywords from this link group in '.
                        'node content and replace them with proper links.'),
  );
  $form['identification']['nodes'] = array(
    '#type' => 'fieldset',
    '#title' => t('Node types'),
    '#collapsible' => TRUE,
  );
  $form['identification']['nodes']['types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Node types'),
    '#options' => node_type_get_names(),
    '#default_value' => $group->types,
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array('#type' => 'submit', '#value' => t('Save'));
  if (isset($group->type)) {
    $form['actions']['delete'] = array('#type' => 'submit', '#value' => t('Delete'));
    $form['type'] = array('#type' => 'value', '#value' => $group->type);
  }

  if (!isset($group->type)) {
    $form['#validate'][] = 'autolink_ui_form_group_new_validate';
  }

  return $form;
}

/**
 * Validate function for add link group form.
 */
function autolink_ui_form_group_new_validate($form, &$form_state) {
  $group = $form_state['values']['name'];
  $result = db_query("SELECT * FROM {autolink_group} WHERE name = :group", array(':group' => $group))->fetchObject();
  if ($result) {
    form_set_error('name', t('A link group with the name %name already exists.', array('%name' => $group)));
  }
}

/**
 * Submit function for add link group form.
 */
function autolink_ui_form_group_submit($form, &$form_state) {
  if ($form_state['clicked_button']['#value'] == t('Delete')) {
    // Rebuild the form to confirm group deletion.
    $form_state['rebuild'] = TRUE;
    $form_state['confirm_delete'] = TRUE;
    return;
  }

  $group = $form_state['group'];
  entity_form_submit_build_entity('autolink_group', $group, $form, $form_state);

  switch (autolink_ui_group_save($group)) {
    case SAVED_NEW:
      drupal_set_message(t('Created new link group %name.', array('%name' => $group->name)));
      watchdog('autolink_ui', 'Created new link group %name.', array('%name' => $group->name), WATCHDOG_NOTICE, l(t('edit'), 'admin/config/content/autolink/' . $group->type . '/edit'));
      break;

    case SAVED_UPDATED:
      drupal_set_message(t('Updated link group %name.', array('%name' => $group->name)));
      watchdog('autolink_ui', 'Updated link group %name.', array('%name' => $group->name), WATCHDOG_NOTICE, l(t('edit'), 'admin/config/content/autolink/' . $group->type . '/edit'));
      break;
  }

  $form_state['values']['type'] = $group->type;
  $form_state['type'] = $group->type;
  $form_state['redirect'] = 'admin/config/content/autolink';
}

/**
 * Page to edit a link group.
 */
function autolink_ui_admin_group_edit($group) {
  if ((isset($_POST['op']) && $_POST['op'] == t('Delete')) || isset($_POST['confirm'])) {
    return drupal_get_form('autolink_ui_group_confirm_delete', $group->type);
  }
  return drupal_get_form('autolink_ui_form_group', (array)$group);
}

/**
 * Form builder for the group delete confirmation form.
 *
 * @ingroup forms
 * @see autolink_ui_group_confirm_delete_submit()
 */
function autolink_ui_group_confirm_delete($form, &$form_state, $vid) {
  $group = autolink_ui_group_load($vid);

  $form['type'] = array('#type' => 'value', '#value' => $vid);

  $form['#group'] = $group;
  $form['#id'] = 'autolink_ui_group_confirm_delete';
  $form['type'] = array('#type' => 'value', '#value' => 'group');
  $form['name'] = array('#type' => 'value', '#value' => $group->name);
  $form['#submit'] = array('autolink_ui_group_confirm_delete_submit');
  return confirm_form($form,
    t('Are you sure you want to delete the link group %title?',
    array('%title' => $group->name)),
    'admin/config/content/autolink',
    t('Deleting a link group will delete all the links in it. This action cannot be undone.'),
    t('Delete'),
    t('Cancel'));
}

/**
 * Submit handler to delete a group after confirmation.
 *
 * @see autolink_ui_group_confirm_delete()
 */
function autolink_ui_group_confirm_delete_submit($form, &$form_state) {
  $status = autolink_ui_group_delete($form_state['values']['type']);
  drupal_set_message(t('Deleted link group %name.', array('%name' => $form_state['values']['name'])));
  watchdog('taxonomy', 'Deleted link group %name.', array('%name' => $form_state['values']['name']), WATCHDOG_NOTICE);
  $form_state['redirect'] = 'admin/config/content/autolink';
  cache_clear_all();
  return;
}

/**
 * Callback for displaying the groups overview form.
 */
function autolink_ui_admin_links($callback_arg = '', $group) {
  module_load_include('inc', 'autolink_ui', 'includes/autolink_ui.operations');
  if (!empty($_POST['links']) && isset($_POST['operation']) && ($_POST['operation'] == 'delete')) {
    $build['autolink_ui_link_multiple_delete_confirm'] = drupal_get_form('autolink_ui_link_multiple_delete_confirm');
  }
  else {
    $build['autolink_ui_overview_links'] = drupal_get_form('autolink_ui_overview_links', $group->type);
  }
  return $build;
}

/**
 * Form builder; Autolink links administration page.
 *
 * @ingroup forms
 * @see autolink_overview_links_validate()
 * @see autolink_overview_links_submit()
 */
function autolink_ui_overview_links($form, &$form_state, $group) {
  $header = array(
    'type' => array('data' => t('Type'), 'field' => 'a.type'),
    'keyword' => array('data' => t('Keyword'), 'field' => 'a.keyword'),
    'destination' => array('data' => t('Destination')),
    'operations' => array('data' => t('Operations')),
  );

  $query = db_select('autolink_link', 'a');
  $query->condition('a.group_type', $group, '=');

  $count_query = clone $query;
  $count_query->addExpression('COUNT(a.lid)');

  $query = $query->extend('PagerDefault')->extend('TableSort');
  $query
    ->fields('a', array('lid', 'group_type', 'type', 'keyword', 'link_path', 'query'))
    ->limit(25)
    ->orderByHeader($header)
    ->setCountQuery($count_query);
  $result = $query->execute();

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#attributes' => array('class' => array('container-inline')),
  );
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#attributes' => array('class' => array('container-inline')),
  );

  // Add our own operations first.
  $ops = autolink_ui_link_operations();
  foreach ($ops as $operation => $array) {
    $options[$operation] = $array['label'];
  }

  // Invoke hook_autolink_operations() to get a list of external operations.
  foreach (module_invoke_all('autolink_link_operations') as $operation => $array) {
    $options[$operation] = $array['label'];
  }

  $form['options']['operation'] = array(
    '#type' => 'select',
    '#title' => t('Operation'),
    '#title_display' => 'invisible',
    '#options' => $options,
    '#default_value' => 'unblock',
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  $destination = drupal_get_destination();

  $options = array();
  $link_types = autolink_ui_link_types();
  foreach ($result as $link) {
    $path = autolink_ui_get_path($link->link_path);
    if ($link->type == 'node_search' || $link->type == 'user_search') {
      $path = $link->link_path .'/'. $link->query;
    }
    $keyword = module_exists('autolink_tracking') ? array('data' => array('#type' => 'link', '#title' => $link->keyword, '#href' => "admin/config/content/autolink/$link->group_type/link/$link->lid")) : check_plain($link->keyword);
    $options[$link->lid] = array(
      'type' => check_plain($link->type),
      'keyword' => $keyword,
      'destination' => check_plain($path),
      'operations' => array('data' => array('#type' => 'link', '#title' => t('edit link'), '#href' => "admin/config/content/autolink/$link->group_type/edit/link/$link->lid", '#options' => array('query' => $destination))),
    );
  }

  $form['links'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No links available.'),
  );
  $form['pager'] = array('#markup' => theme('pager'));
  $form['group'] = array('#type' => 'value', '#value' => $group);

  return $form;
}

/**
 * Form validation for Autolink administration update form.
 */
function autolink_ui_overview_links_validate($form, &$form_state) {
  $form_state['values']['links'] = array_filter($form_state['values']['links']);
  if (!count($form_state['values']['links'])) {
    form_set_error('', t('No link definitions were selected.'));
  }
}

/**
 * Submit handler for Autolink administration update form.
 */
function autolink_ui_overview_links_submit($form, &$form_state) {
  $operations = autolink_ui_link_operations($form_state);
  $operations += module_invoke_all('autolink_link_operations', $form_state);
  $operation = $operations[$form_state['values']['operation']];
  // Filter out unchecked links.
  $links = array_filter($form_state['values']['links']);
  if ($function = $operation['callback']) {
    // Add in callback arguments if present.
    if (isset($operation['callback arguments'])) {
      $args = array_merge(array($links), $operation['callback arguments']);
    }
    else {
      $args = array($links);
    }
    // Call the function based on the operation that was selected.
    call_user_func_array($function, $args);

    drupal_set_message(t('The update has been performed.'));
  }
}

/**
 * Form for adding or editing a link definition.
 *
 * This function uses the AHAH Helper module to build a dynamic link definition
 * form. When the AHAH submit function is called and the form is rebuilt this
 * function builds link type specific form elements based on data retrieved with
 * the link type that has been selected.
 */
function autolink_ui_form_link($form, &$form_state, $group, $edit = array()) {
  _autolink_ui_include();

  $form = array();
  $form['#group'] = (array)$group;

  $keyword_exists = FALSE;
  $keyword_required = TRUE;

  // If this is an edit form then load the link data and store the link object
  // in the form so other modules can access it using hook_form_alter().
  if (isset($edit['lid'])) {
    $link = autolink_ui_link_load($edit['lid']);
    $form['link'] = array('#type' => 'value', '#value' => $link);
    $keyword_exists = TRUE;
    $keyword_required = FALSE;
  }

  if (empty($form_state['values']['operation'])) {
    $selected_method = isset($edit['method']) ? $edit['method'] : 'select';
  }
  else {
    $selected_method = isset($form_state['values']['operation']) ? $form_state['values']['operation'] : 'select';
  }

  // If there is only one link type then set that as the default instead of 'select'.
  $method_options['select'] = t('--Selection method--');
  $method_options += autolink_ui_form_methods();
  $form['operation'] = array(
    '#type' => 'select',
    '#title' => t('Selection method'),
    '#options' => $method_options,
    '#default_value' => $selected_method,
    '#ajax' => array(
      'event' => 'change',
      'callback' => 'autolink_ui_build_link_form',
      'wrapper' => 'link-wrapper',
      'effect' => 'fade',
    ),
  );

  $form['link_wrapper'] = array(
    '#prefix' => '<div id="link-wrapper">',
    '#suffix' => '</div>',
    '#tree'   => TRUE,
  );

  // Only if a link type has been selected do we show the link definition form.
  if (!empty($selected_method) && $selected_method != 'select') {
    $form['link_wrapper']['definition'] = array(
      '#type' => 'fieldset',
      '#title' => t('Definition'),
    );
    $form['link_wrapper']['definition']['keyword'] = array(
      '#type' => 'textfield',
      '#title' => t('Keyword(s)'),
      '#default_value' => $edit['keyword'],
      '#required' => $keyword_required,
      '#description' => t('The keyword you would like converted into a link. This can be more than one word'),
    );
    $form['link_wrapper']['definition']['case_sensitive'] = array(
      '#type' => 'checkbox',
      '#title' => t('Case-sensitive keyword matching'),
      '#default_value' => isset($edit['case_sensitive']) ? $edit['case_sensitive'] : variable_get('autolink_case_sensitivity', 0),
      '#description' => t('Words will only be matched if they match letter case. i.e. <i>AutoLink</i>' .
                          'is not the same as <i>autolink</i> if case-sensitivity is enabled.'),
    );

    // We don't want the keyword to be edited once it is stored in the database
    // since we search for those keywords in nodes on update if the path is changed.
    // Since Drupal will not save the default value of 'keyword' while it is
    // disabled we have to create a hidden field to store our value.
    if ($keyword_exists) {
      $form['link_wrapper']['old_keyword'] = array(
        '#type' => 'value',
        '#value' => $edit['keyword'],
      );
    }

    // Prepare callbacks and arguments to build form elements based on the selection method.
    $method = _autolink_ui_get_method($selected_method);
    $form['method'] = array('#type' => 'value', '#value' => $selected_method);
    $method['callback arguments'] = isset($method['callback arguments']) ? $method['callback arguments'] : array();

    // Call the selection method specific form builder callback.
    $type_form = _autolink_ui_method_form_preprocess($method['callback'], $edit, $method['callback arguments']);
    if ($type_form) {
      $form['link_wrapper']['definition'] += $type_form;
    }
  }

  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));
  $form['group_type'] = array('#type' => 'value', '#value' => $group->type);
  if (isset($edit['lid'])) {
    $form['delete'] = array('#type' => 'submit', '#value' => t('Delete'));
    $form['lid'] = array('#type' => 'value', '#value' => $edit['lid']);
  }

  return $form;
}

/**
 * AJAX callback for the link add/edit form.
 */
function autolink_ui_build_link_form($form, $form_state) {
  return $form['link_wrapper'];
}

/**
 * Validation handler for the link edit form. 
 *
 * Here we ensure the link does not contain inappropriate characters. Also, since
 * we are using disabled and hidden fields, we have to do extra form validation
 * here because we cannot use the 'required' => TRUE property on a disabled field.
 *
 * @see autolink_ui_form_link()
 */
function autolink_ui_form_link_validate($form, &$form_state) {
  _autolink_ui_include();

  $form_values = _autolink_ui_merge_link_form_values($form_state['values']);

  if (empty($form_values['keyword']) && !isset($form_values['stored_keyword'])) {
    form_set_error('keyword', t('You must enter a keyword.'));
  }
  elseif (!empty($form_values['keyword']) && preg_match('!^[^a-zA-Z0-9]+$!', $form_values['keyword']) == 1) {
    form_set_error('keyword', t('The keyword contains an unacceptable value.'));
  }

  // Call validate callbacks specified by the selection method.
  $method = _autolink_ui_get_method($form_values['method']);
  if (function_exists($method['validate'])) {
    $values['form_values'] = $form_values;
    if (isset($method['callback arguments'])) {
      $values += $method['callback arguments'];
    }
    call_user_func_array($method['validate'], $values);
  }
}

/**
 * Final submit function for the link form.
 */
function autolink_ui_form_link_submit($form, &$form_state) {
  $link = _autolink_ui_merge_link_form_values($form_state['values']);

  switch (autolink_ui_link_save($link)) {
    case SAVED_NEW:
      drupal_set_message(t('Created new <i>%link_type</i> type link for keyword <i>%keyword</i>.', array('%link_type' => $link['type'], '%keyword' => $link['keyword'])));
      watchdog('Autolink', 'Created new <i>%link_type</i> type link for keyword <i>%keyword</i>.', array('%link_type' => $link['type'], '%keyword' => $link['keyword']), WATCHDOG_NOTICE, l(t('edit'), 'admin/content/autolink/$link->group_type/edit/link/'. $link['lid']));
      break;
    case SAVED_UPDATED:
      drupal_set_message(t('Updated <i>%link_type</i> type link for keyword <i>%keyword</i>.', array('%link_type' => $form_state['values']['link_wrapper']['type'], '%keyword' => $link['stored_keyword'])));
      watchdog('Autolink', 'Updated link for keyword <i>%keyword</i>.', array('%keyword' => $link['stored_keyword']), WATCHDOG_NOTICE, l(t('edit'), 'admin/content/autolink/'. $link['group_type'] .'/edit/link/'. $link['lid']));
      break;
  }

  drupal_goto('admin/config/content/autolink/'. $link['group_type']);

  return;
}

/**
 * Page to add a link.
 */
function autolink_ui_add_link_page($group) {
  drupal_set_title(t("Add link definition to $group->name"));
  return drupal_get_form('autolink_ui_form_link', $group);
}

/**
 * Page to edit a link.
 *
 * @param $link
 *   A link object.
 *
 * @see autolink_form_link()
 * @see autolink_confirm_delete()
 */
function autolink_ui_admin_link_edit($group, $link) {
  if ((isset($_POST['op']) && $_POST['op'] == t('Delete')) || isset($_POST['confirm'])) {
    return drupal_get_form('autolink_ui_confirm_link_delete', $link->lid);
  }
    return drupal_get_form('autolink_ui_form_link', $group, (array)$link);
}

/**
 * Form builder for the link delete confirmation form.
 *
 * @param $lid
 *   The ID of a link.
 *
 * @ingroup forms
 * @see autolink_confirm_link_delete_submit()
 */
function autolink_ui_confirm_link_delete($form, &$form_state, $lid) {
  $link = autolink_ui_link_load($lid);

  $form['type'] = array(
    '#type' => 'value',
    '#value' => 'link',
  );
  $form['lid'] = array(
    '#type' => 'value',
    '#value' => $link->lid,
  );
  $form['keyword'] = array(
    '#type' => 'value',
    '#value' => $link->keyword,
  );
  return confirm_form($form,
                  t('Are you sure you want to delete the link %keyword?',
                  array('%keyword' => $link->keyword)),
                  "admin/content/autolink/$link->group_type",
                  t('This action cannot be undone.'),
                  t('Delete'),
                  t('Cancel'));
}

/**
 * Submit handler to delete a link after confirmation.
 *
 * @see autolink_confirm_link_delete()
 */
function autolink_ui_confirm_link_delete_submit($form, &$form_state) {
  // Delete the link and record the deletion
  $status = autolink_ui_link_delete($form_state['values']['lid']);

  drupal_set_message(t('Deleted link definition for keyword <i>%link</i>.', array('%link' => $form_state['values']['keyword'])));
  watchdog('Autolink', 'Deleted link definition for keyword <i>%link</i>.', array('%link' => $form_state['values']['keyword']), WATCHDOG_NOTICE);
  $form_state['redirect'] = 'admin/content/autolink';

  return;
}

/**
 * Invokes hook_autolink_LINK_TYPE_form_elements() to retrieve form elements for the link form.
 *
 * @param $callback
 *   A string representing the current selection method.
 * @param $edit = array()
 *   Form values that need to be set as default values on form elements.
 * @param $callback_arguments = array()
 *   An optional array of arguments to pass to the form callback function.
 *
 * @return
 *   Form elements populated with defaults and ready for display.
 */
function _autolink_ui_method_form_preprocess($callback, $edit = array(), $callback_arguments = array()) { 
  $edit['argument'] = isset($edit['link_path']) ? html_entity_decode($edit['link_path']) : '';
  $arguments['edit'] = $edit;
  if (!empty($callback_arguments)) {
    $arguments += $callback_arguments;
  }
  foreach (call_user_func_array($callback, $arguments) as $field => $value) {
    $elements[$field] = $value;
  }
  return $elements;
}

/**
 * Merges AJAX form values into a readable $link array in preparation
 * for form validation and database inserts.
 *
 * @return
 *   Merged form values that can be processed for validation or inserted into the database.
 */
function _autolink_ui_merge_link_form_values($form_values) {
  $link = $form_values;
  if (isset($form_values['link_wrapper']['definition'])) {
    $link += $form_values['link_wrapper']['definition'];
  }
  return $link;
}
