<?php

/**
 * @file
 * Functions for the 'link' link type for use with the Autolink module.
 */

/**
 * Form builder callback for url method.
 */
function autolink_ui_link_method($edit) {
  $form['external_url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#required' => TRUE,
    '#default_value' => isset($edit['link_path']) ? $edit['link_path'] : '',
    '#description' => t('e.g. <i>http://www.google.com</i>'),
  );
  return $form;
}

/**
 * Validation callback for url form elements..
 */
function autolink_ui_link_method_validate($form_values) {
  // Validate the url as either an external path.
  if (!url_is_external($form_values['external_url'])) {
    form_set_error('external_url', t('The url \'%url\' is invalid. You must provide a valid external url.', array('%url' => $form_values['argument'])));
  }
}

/**
 * Implements hook_autolink_METHOD_alter().
 *
 * Builds a value for the link_path field.
 *
 * @param $link
 *   An array of form data for the link definition.
 *
 * @return
 *   The modified path to be saved to the database.
 */
function autolink_ui_autolink_link_alter(&$link, $method) {
  $link['link_path'] = $link['external_url'];
}
