<?php

/**
 * @file
 * Functions for the 'user' link type for use with the Autolink module.
 */

/**
 * Callback function for building the user link form.
 */
function autolink_ui_user_method($edit, $type) {
  $form = array();
  if (isset($edit['link_path'])) {
    $args = explode('/', $edit['link_path']);
    $uid = $args[1];
    $user = is_numeric($uid) ? user_load($uid) : NULL;
  }

  switch ($type) {
    case 'select':
      $users = autolink_ui_user_get_users();

      $select['select'] = t('--Select a user--');
      $select += $users;

      $form['user_select'] = array(
        '#type' => 'select',
        '#title' => t('Users'),
        '#required' => TRUE,
        '#options' => $select,
        '#default_value' => isset($uid) ? $uid : 'select',
        '#description' => t('Links will link to the user profile of the selected user.'),
      );
      break;
    case 'name':
      $form['user_name'] = array(
        '#type' => 'textfield',
        '#title' => t('Enter a username'),
        '#required' => TRUE,
        '#default_value' => isset($user) && !is_null($user) ? $user->name : '',
      );
      break;
    case 'mail':
      $form['user_mail'] = array(
        '#type' => 'textfield',
        '#title' => t('Enter a user email'),
        '#required' => TRUE,
        '#default_value' => isset($user) && !is_null($user) ? $user->mail : '',
      );
      break;
  }

  return $form;
}

/**
 * Validation callback for user link form elements.
 */
function autolink_ui_user_method_validate($form_values, $type) {
  switch ($type) {
    case 'select':
      if ($form_values['user_select'] == 'select') {
        form_set_error('user_select', t('You must select a user.'));
      }
      break;
    case 'name':
      $user = db_query('SELECT uid FROM {user} WHERE name = :name', array(':name' => $form_values['user_name']))->fetchObject;
      if (!$user) {
       form_set_error('user_name', t('The user <i>%user</i> does not exist.', array('%user' => $form_values['user_name'])));
      }
      break;
    case 'mail':
      $user = db_query('SELECT uid FROM {user} WHERE mail = :mail', array(':mail' => $form_values['user_mail']))->fetchObject;
      if (!$user) {
        form_set_error('user_mail', t('Cannot locate user data for user email \'%email\'.', array('%email' => $form_values['user_mail'])));
      }
      break;
  }
}

/**
 * Implements hook_autolink_LINK_TYPE_alter().
 *
 * Converts the link path to correctly formatted user path. Form data is used to
 * create a proper link path linking to the results page.
 *
 * @param $link
 *   An associative array of link form data.
 *
 * @see autolink_ui_link_save()
 */
function autolink_ui_autolink_user_alter(&$link, $method) {
  switch ($method) {
    case 'user_select':
      $link['link_path'] = 'user/'. $link['user_select'];
      break;
    case 'user_name':
      $user = db_query('SELECT * FROM {user} WHERE name = :name', array(':name' => $link['user_name']))->fetchObject();
      $link['link_path'] = 'user/'. $user->uid;
      break;
    case 'user_mail':
      $user = db_query('SELECT * FROM {user} WHERE name = :name', array(':name' => $link['user_mail']))->fetchObject();
      $link['link_path'] = 'user/'. $user->uid;
      break;
  }
}

/**
 * Saves a new link definition based on user data.
 *
 * @see autolink_ui_user_operations()
 * @see autolink_ui_user_operations_name()
 */
function autolink_ui_user_save_link($user, $group) {
  $link['type'] = 'user';
  $link['group_type'] = $group;
  $link['keyword'] = $user->name;
  $link['link_path'] = 'user/'. $user->uid;
  $link['method'] = 'user_name';
  $link['case_sensitive'] = variable_get('autolink_case_sensitivity', 0);

  drupal_write_record('autolink', $link);
}

/**
 * Returns a list of users.
 *
 * @see autolink_ui_autolink_user_form_elements()
 */
function autolink_ui_user_get_users() {
  $result = db_query('SELECT u.uid, u.name FROM {users} u WHERE u.uid <> 0');
  foreach ($result as $user) {
    $users[$user->uid] = $user->name;
  }
  asort($users);
  return $users;
}
